@@ -37,6 +37,7 @@ from robotoff.models import (
 from robotoff.off import (
     OFFAuthentication,
     generate_image_path,
+    get_barcode_from_url,
     get_product,
     get_server_type,
 )
@@ -319,9 +320,14 @@ class NutrientPredictorResource:
         if not ocr_url.endswith(".json"):
             raise falcon.HTTPBadRequest("a JSON file is expected")
 
+        barcode = get_barcode_from_url(ocr_url)
+
+        if barcode is None:
+            raise falcon.HTTPBadRequest(f"invalid OCR URL: {ocr_url}")
+
         try:
-            product_predictions = extract_ocr_predictions(
-                ocr_url, [PredictionType.nutrient]
+            predictions = extract_ocr_predictions(
+                barcode, ocr_url, [PredictionType.nutrient]
             )
 
         except requests.exceptions.RequestException:
@@ -339,21 +345,20 @@ class NutrientPredictorResource:
             }
             return
 
-        if not product_predictions:
-            resp.media = {
-                "nutrients": {},
-            }
-        else:
-            nutrient_predictions = product_predictions[PredictionType.nutrient]
-            resp.media = nutrient_predictions.to_dict()
+        resp.media = {"nutrients": [p.to_dict() for p in predictions]}
 
 
 class OCRInsightsPredictorResource:
     def on_get(self, req: falcon.Request, resp: falcon.Response):
         ocr_url = req.get_param("ocr_url", required=True)
+        barcode = get_barcode_from_url(ocr_url)
+        if barcode is None:
+            raise falcon.HTTPBadRequest(f"invalid OCR URL: {ocr_url}")
 
         try:
-            insights = extract_ocr_predictions(ocr_url, DEFAULT_OCR_PREDICTION_TYPES)
+            insights = extract_ocr_predictions(
+                barcode, ocr_url, DEFAULT_OCR_PREDICTION_TYPES
+            )
 
         except requests.exceptions.RequestException:
             resp.media = {
@@ -384,18 +389,10 @@ class CategoryPredictorResource:
 
         product = get_product(barcode)
         if product:
-            predicted = CategoryClassifier(
+            predictions = CategoryClassifier(
                 get_taxonomy(TaxonomyType.category.name)
             ).predict(product, deepest_only)
-
-            if predicted:
-                categories = [
-                    {
-                        "category": prediction.category,
-                        "confidence": prediction.confidence,
-                    }
-                    for prediction in predicted
-                ]
+            categories = [p.to_dict() for p in predictions]
 
         resp.media = {"categories": categories}
 
@@ -450,10 +447,10 @@ class ImageCropResource:
         x_min = req.get_param_as_float("x_min", required=True)
         y_max = req.get_param_as_float("y_max", required=True)
         x_max = req.get_param_as_float("x_max", required=True)
-        image = get_image_from_url(image_url)
+        image = get_image_from_url(image_url, session=http_session, error_raise=False)
 
         if image is None:
-            raise falcon.HTTPBadRequest("invalid image")
+            raise falcon.HTTPBadRequest(f"Could not fetch image: {image_url}")
 
         (left, right, top, bottom) = (
             x_min * image.width,
@@ -564,11 +561,10 @@ class ImagePredictorResource:
                 "when `output_image` is True",
             )
 
-        image = get_image_from_url(image_url, session=http_session)
+        image = get_image_from_url(image_url, session=http_session, error_raise=False)
 
         if image is None:
-            logger.info("Could not fetch image: {}".format(image_url))
-            return
+            raise falcon.HTTPBadRequest(f"Could not fetch image: {image_url}")
 
         predictions = {}
 
@@ -749,7 +745,6 @@ class ImageLogoAnnotateResource:
             type_ = annotation["type"]
             value = annotation["value"] or None
             logo = LogoAnnotation.get_by_id(logo_id)
-
             if value is not None:
                 logo.annotation_value = value
                 value_tag = get_tag(value)
@@ -762,7 +757,8 @@ class ImageLogoAnnotateResource:
             logo.save()
             annotated_logos.append(logo)
 
-        generate_insights_from_annotated_logos(annotated_logos, server_domain)
+        created = generate_insights_from_annotated_logos(annotated_logos, server_domain)
+        resp.media = {"created insights": created}
 
 
 class ImageLogoUpdateResource:
