@@ -1,16 +1,18 @@
+import json
 import pathlib
-from typing import Dict, Iterable, List, Optional, TextIO, Tuple, Union
+from typing import Callable, Dict, Iterable, List, Optional, TextIO, Tuple, Union
 
 import orjson
+import requests
 
-from robotoff.off import generate_json_ocr_url, split_barcode
+from robotoff.off import generate_json_ocr_url, get_barcode_from_path, split_barcode
 from robotoff.prediction.types import Prediction, PredictionType
 from robotoff.settings import BaseURLProvider
 from robotoff.utils import get_logger, http_session, jsonl_iter, jsonl_iter_fp
 from robotoff.utils.types import JSONType
 
 from .brand import find_brands
-from .dataclass import OCRResult
+from .dataclass import OCRParsingException, OCRResult, OCRResultGenerationException
 from .expiration_date import find_expiration_date
 from .image_flag import flag_image
 from .image_lang import get_image_lang
@@ -27,16 +29,24 @@ from .trace import find_traces
 logger = get_logger(__name__)
 
 
-def get_barcode_from_path(path: str) -> Optional[str]:
-    barcode = ""
-
-    for parent in pathlib.Path(path).parents:
-        if parent.name.isdigit():
-            barcode = parent.name + barcode
-        else:
-            break
-
-    return barcode or None
+PREDICTION_TYPE_TO_FUNC: Dict[
+    str, Callable[[Union[OCRResult, str]], List[Prediction]]
+] = {
+    PredictionType.packager_code: find_packager_codes,
+    PredictionType.label: find_labels,
+    PredictionType.expiration_date: find_expiration_date,
+    PredictionType.image_flag: flag_image,
+    PredictionType.image_orientation: find_image_orientation,
+    PredictionType.product_weight: find_product_weight,
+    PredictionType.trace: find_traces,
+    PredictionType.nutrient: find_nutrient_values,
+    PredictionType.nutrient_mention: find_nutrient_mentions,
+    PredictionType.brand: find_brands,
+    PredictionType.store: find_stores,
+    PredictionType.packaging: find_packaging,
+    PredictionType.location: find_locations,
+    PredictionType.image_lang: get_image_lang,
+}
 
 
 def fetch_images_for_ean(ean: str):
@@ -55,54 +65,61 @@ def get_json_for_image(barcode: str, image_id: str) -> Optional[JSONType]:
     return r.json()
 
 
+def get_ocr_result(
+    ocr_url: str, session: requests.Session, error_raise: bool = True
+) -> Optional[OCRResult]:
+    try:
+        r = session.get(ocr_url)
+    except requests.exceptions.RequestException as e:
+        error_message = f"HTTP Error when fetching OCR URL {ocr_url}"
+        if error_raise:
+            raise OCRResultGenerationException(error_message) from e
+
+        logger.warning(error_message, exc_info=e)
+        return None
+
+    try:
+        ocr_data: Dict = r.json()
+    except json.JSONDecodeError as e:
+        error_message = f"Error while decoding OCR JSON from {ocr_url}"
+        if error_raise:
+            raise OCRResultGenerationException(error_message) from e
+
+        logger.warning(error_message, exc_info=e)
+        return None
+
+    try:
+        return OCRResult.from_json(ocr_data)
+    except OCRParsingException as e:
+        if error_raise:
+            raise OCRResultGenerationException(str(e)) from e
+
+        logger.warning(f"Error while parsing OCR JSON from {ocr_url}", exc_info=e)
+        return None
+
+
 def extract_predictions(
-    content: Union[OCRResult, str], prediction_type: PredictionType
+    content: Union[OCRResult, str],
+    prediction_type: PredictionType,
+    barcode: Optional[str] = None,
+    source_image: Optional[str] = None,
 ) -> List[Prediction]:
-    """Proxy to each predictor, depending on prediction type."""
-    if prediction_type == PredictionType.packager_code:
-        return find_packager_codes(content)
-
-    elif prediction_type == PredictionType.label:
-        return find_labels(content)
-
-    elif prediction_type == PredictionType.expiration_date:
-        return find_expiration_date(content)
-
-    elif prediction_type == PredictionType.image_flag:
-        return flag_image(content)
-
-    elif prediction_type == PredictionType.image_orientation:
-        return find_image_orientation(content)
-
-    elif prediction_type == PredictionType.product_weight:
-        return find_product_weight(content)
-
-    elif prediction_type == PredictionType.trace:
-        return find_traces(content)
-
-    elif prediction_type == PredictionType.nutrient:
-        return find_nutrient_values(content)
-
-    elif prediction_type == PredictionType.nutrient_mention:
-        return find_nutrient_mentions(content)
-
-    elif prediction_type == PredictionType.brand:
-        return find_brands(content)
-
-    elif prediction_type == PredictionType.store:
-        return find_stores(content)
-
-    elif prediction_type == PredictionType.packaging:
-        return find_packaging(content)
-
-    elif prediction_type == PredictionType.location:
-        return find_locations(content)
-
-    elif prediction_type == PredictionType.image_lang:
-        return get_image_lang(content)
+    """Extract predictions from OCR using for provided prediction type.
 
+    :param content: OCR output to extract predictions from.
+    :param barcode: Barcode to add to each prediction, defaults to None.
+    :param source_image: `source_image`to add to each prediction, defaults to
+    None.
+    :return: The generated predictions.
+    """
+    if prediction_type in PREDICTION_TYPE_TO_FUNC:
+        predictions = PREDICTION_TYPE_TO_FUNC[prediction_type](content)
+        for prediction in predictions:
+            prediction.barcode = barcode
+            prediction.source_image = source_image
+        return predictions
     else:
-        raise ValueError("unknown prediction type: {}".format(prediction_type))
+        raise ValueError(f"unknown prediction type: {prediction_type}")
 
 
 def is_barcode(text: str):
