@@ -11,7 +11,6 @@ from robotoff.insights.importer import import_insights, refresh_insights
 from robotoff.models import with_db
 from robotoff.off import ServerType, get_server_type
 from robotoff.prediction.category.neural.category_classifier import CategoryClassifier
-from robotoff.prediction.types import PredictionType, ProductPredictions
 from robotoff.products import get_product
 from robotoff.taxonomy import TaxonomyType, get_taxonomy
 from robotoff.utils import get_logger
@@ -53,15 +52,15 @@ def add_category_insight(barcode: str, product: JSONType, server_domain: str) ->
     logger.info("Predicting product categories...")
     # predict category using Elasticsearch on title
     product_predictions = []
-    product_insight = predict_category_from_product_es(product)
+    es_prediction = predict_category_from_product_es(product)
 
-    if product_insight is not None:
-        product_predictions.append(product_insight)
+    if es_prediction is not None:
+        product_predictions.append(es_prediction)
 
     # predict category using neural model
-    category_predictions = None
+    neural_predictions = []
     try:
-        category_predictions = CategoryClassifier(
+        neural_predictions = CategoryClassifier(
             get_taxonomy(TaxonomyType.category.name)
         ).predict(product)
     except requests.exceptions.HTTPError as e:
@@ -70,25 +69,14 @@ def add_category_insight(barcode: str, product: JSONType, server_domain: str) ->
             f"Category classifier returned an error: {resp.status_code}: {resp.text}"
         )
 
-    if category_predictions is not None:
-        product_predictions.append(
-            ProductPredictions(
-                barcode=product["code"],
-                type=PredictionType.category,
-                predictions=[
-                    category_prediction.to_prediction()
-                    for category_prediction in category_predictions
-                ],
-            )
-        )
+    for neural_prediction in neural_predictions:
+        neural_prediction.barcode = barcode
+        product_predictions.append(neural_prediction)
 
     if len(product_predictions) < 1:
         return False
 
-    merged_product_prediction = ProductPredictions.merge(product_predictions)
-    imported = import_insights(
-        [merged_product_prediction], server_domain, automatic=True
-    )
+    imported = import_insights(product_predictions, server_domain, automatic=True)
     logger.info(f"{imported} category insight imported for product {barcode}")
 
     return bool(imported)
@@ -105,9 +93,7 @@ def updated_product_predict_insights(
 
     logger.info("Generating predictions from product name...")
     predictions_all = get_predictions_from_product_name(barcode, product_name)
-    imported = import_insights(
-        list(predictions_all.values()), server_domain, automatic=False
-    )
+    imported = import_insights(predictions_all, server_domain, automatic=False)
     logger.info(f"{imported} insights imported for product {barcode}")
 
     if imported:
